#coding: utf8
import plask


class Diffusion2D(plask.Solver):
    """
    Diffusion2D(name="")
    
    Calculates carrier pairs concentration in active region using FEM in two-dimensional Cartesian space
    """
    algorithm = None
    """
    Chosen matrix factorization algorithm
    """

    def compute(self, loops=0, shb=False, reg=None):
        """
        compute(self, loops=0, shb=False, reg=None)
        
            Run diffusion calculations
            
            Args:
                loops (int): Number of iterations to perform. If 0, the solver will run
                             until the convergence.
                shb (bool): If True, the solver will use take into account the spatial hole
                            burning effect.
                reg (int or None): Index of the active region to compute. If None, perform
                                   computations for all the active regions.
        """

    def compute_overthreshold(self):
        """
        compute_overthreshold(self)
        
            Deprecated method. Use compute(shb=True) instead.
        """

    def compute_threshold(self):
        """
        compute_threshold(self)
        
            Deprecated method. Use compute() instead.
        """

    geometry = None
    """
    Geometry provided to the solver
    """

    def get_burning_for_mode(self, mode):
        """
        get_burning_for_mode(self, mode)
        
            Get power burned over threshold by specified mode [mW].
        """

    def get_total_burning(self):
        """
        get_total_burning(self)
        
            Get total power burned over threshold [mW].
        """

    id = None
    """
    Id of the solver object. (read only)
    
    Example:
        >>> mysolver.id
        mysolver:category.type
    """

    inCurrentDensity = None
    """
    Receiver of the current density required for computations [kA/cm²].
    
    
    You will find usage details in the documentation of the receiver class
    :class:`~plask.flow.CurrentDensityReceiver2D`.
    
    Example:
       Connect the reveiver to a provider from some other solver:
    
       >>> solver.inCurrentDensity = other_solver.outCurrentDensity
    
    See also:
    
       Receciver class: :class:`plask.flow.CurrentDensityReceiver2D`
    
       Provider class: :class:`plask.flow.CurrentDensityProvider2D`
    
       Data filter: :class:`plask.filter.CurrentDensityFilter2D`
    """

    inGain = None
    """
    Receiver of the material gain required for computations [1/cm].
    It is required only for the SHB computations.
    
    You will find usage details in the documentation of the receiver class
    :class:`~plask.flow.GainReceiver2D`.
    
    Example:
       Connect the reveiver to a provider from some other solver:
    
       >>> solver.inGain = other_solver.outGain
    
    See also:
    
       Receciver class: :class:`plask.flow.GainReceiver2D`
    
       Provider class: :class:`plask.flow.GainProvider2D`
    
       Data filter: :class:`plask.filter.GainFilter2D`
    """

    inLightE = None
    """
    Receiver of the electric field required for computations [V/m].
    It is required only for the SHB computations.
    
    You will find usage details in the documentation of the receiver class
    :class:`~plask.flow.ModeLightEReceiver2D`.
    
    Example:
       Connect the reveiver to a provider from some other solver:
    
       >>> solver.inLightE = other_solver.outModeLightE
    
    See also:
    
       Receciver class: :class:`plask.flow.ModeLightEReceiver2D`
    
       Provider class: :class:`plask.flow.ModeLightEProvider2D`
    
       Data filter: :class:`plask.filter.ModeLightEFilter2D`
    """

    inTemperature = None
    """
    Receiver of the temperature required for computations [K].
    
    
    You will find usage details in the documentation of the receiver class
    :class:`~plask.flow.TemperatureReceiver2D`.
    
    Example:
       Connect the reveiver to a provider from some other solver:
    
       >>> solver.inTemperature = other_solver.outTemperature
    
    See also:
    
       Receciver class: :class:`plask.flow.TemperatureReceiver2D`
    
       Provider class: :class:`plask.flow.TemperatureProvider2D`
    
       Data filter: :class:`plask.filter.TemperatureFilter2D`
    """

    inWavelength = None
    """
    Receiver of the wavelength required for computations [nm].
    It is required only for the SHB computations.
    
    You will find usage details in the documentation of the receiver class
    :class:`~plask.flow.ModeWavelengthReceiver`.
    
    Example:
       Connect the reveiver to a provider from some other solver:
    
       >>> solver.inWavelength = other_solver.outModeWavelength
    
    See also:
    
       Receciver class: :class:`plask.flow.ModeWavelengthReceiver`
    
       Provider class: :class:`plask.flow.ModeWavelengthProvider`
    
       Data filter: :class:`plask.filter.ModeWavelengthFilter`
    """

    def initialize(self):
        """
        initialize(self)
        
            Initialize solver.
            
            This method manually initialized the solver and sets :attr:`initialized` to
            *True*. Normally calling it is not necessary, as each solver automatically
            initializes itself when needed.
            
            Returns:
                bool: solver :attr:`initialized` state prior to this method call.
        """

    initialized = None
    """
    True if the solver has been initialized. (read only)
    
    Solvers usually get initialized at the beginning of the computations.
    You can clean the initialization state and free the memory by calling
    the :meth:`invalidate` method.
    """

    def invalidate(self):
        """
        invalidate(self)
        
            Set the solver back to uninitialized state.
            
            This method frees the memory allocated by the solver and sets
            :attr:`initialized` to *False*.
        """

    iterative = None
    """
    Iterative matrix parameters (see :py:class:`~plask.IterativeParams`)
    """

    def load_xpl(self, xpl, manager):
        """
        load_xpl(self, xpl, manager)
        
            Load configuration from XPL reader.
            
            This method should be overriden in custom Python solvers.
            
            Example:
              .. code-block:: python
            
                 def load_xpl(self, xpl, manager):
                     for tag in xpl:
                         if tag == 'config':
                             self.a = tag['a']
                             self.b = tag.get('b', 0)
                             if 'c' in tag:
                                 self.c = tag['c']
                         if tag == 'combined':
                             for subtag in tag:
                                 if subtag == 'withtext':
                                     self.data = subtag.attrs
                                     # Text must be read last
                                     self.text = subtag.text
                         elif tag == 'geometry':
                             self.geometry = tag.getitem(manager.geo, 'ref')
        """

    maxerr = None
    """
    Maximum relative residual error (%)
    """

    mesh = None
    """
    Mesh provided to the solver
    """

    outCarriersConcentration = None
    """
    Provider of the computed carriers concentration [1/cm³].
    
    
    outCarriersConcentration(n=0, mesh, interpolation='default')
    
    :param str type: Detailed information which carriers are returned. It can be
                     'majority' to return majority carriers in given material,
                     'pairs' for the concentration of electron-hole pairs,
                     'electrons', or 'holes' for particular carriers type.
    :param mesh mesh: Target mesh to get the field at.
    :param str interpolation: Requested interpolation method.
    
    :return: Data with the carriers concentration on the specified mesh **[1/cm³]**.
    
    You may obtain the number of different values this provider can return by
    testing its length.
    
    Example:
       Connect the provider to a receiver in some other solver:
    
       >>> other_solver.inCarriersConcentration = solver.outCarriersConcentration
    
       Obtain the provided field:
    
       >>> solver.outCarriersConcentration(0, mesh)
       <plask.Data at 0x1234567>
    
       Test the number of provided values:
    
       >>> len(solver.outCarriersConcentration)
       3
    
    See also:
    
       Provider class: :class:`plask.flow.CarriersConcentrationProvider2D`
    
       Receciver class: :class:`plask.flow.CarriersConcentrationReceiver2D`
    """



class Diffusion3D(plask.Solver):
    """
    Diffusion3D(name="")
    
    Calculates carrier pairs concentration in active region using FEM in three-dimensional space
    """
    algorithm = None
    """
    Chosen matrix factorization algorithm
    """

    def compute(self, loops=0, shb=False, reg=None):
        """
        compute(self, loops=0, shb=False, reg=None)
        
            Run diffusion calculations
            
            Args:
                loops (int): Number of iterations to perform. If 0, the solver will run
                             until the convergence.
                shb (bool): If True, the solver will use take into account the spatial hole
                            burning effect.
                reg (int or None): Index of the active region to compute. If None, perform
                                   computations for all the active regions.
        """

    geometry = None
    """
    Geometry provided to the solver
    """

    def get_burning_for_mode(self, mode):
        """
        get_burning_for_mode(self, mode)
        
            Get power burned over threshold by specified mode [mW].
        """

    def get_total_burning(self):
        """
        get_total_burning(self)
        
            Get total power burned over threshold [mW].
        """

    id = None
    """
    Id of the solver object. (read only)
    
    Example:
        >>> mysolver.id
        mysolver:category.type
    """

    inCurrentDensity = None
    """
    Receiver of the current density required for computations [kA/cm²].
    
    
    You will find usage details in the documentation of the receiver class
    :class:`~plask.flow.CurrentDensityReceiver3D`.
    
    Example:
       Connect the reveiver to a provider from some other solver:
    
       >>> solver.inCurrentDensity = other_solver.outCurrentDensity
    
    See also:
    
       Receciver class: :class:`plask.flow.CurrentDensityReceiver3D`
    
       Provider class: :class:`plask.flow.CurrentDensityProvider3D`
    
       Data filter: :class:`plask.filter.CurrentDensityFilter3D`
    """

    inGain = None
    """
    Receiver of the material gain required for computations [1/cm].
    It is required only for the SHB computations.
    
    You will find usage details in the documentation of the receiver class
    :class:`~plask.flow.GainReceiver3D`.
    
    Example:
       Connect the reveiver to a provider from some other solver:
    
       >>> solver.inGain = other_solver.outGain
    
    See also:
    
       Receciver class: :class:`plask.flow.GainReceiver3D`
    
       Provider class: :class:`plask.flow.GainProvider3D`
    
       Data filter: :class:`plask.filter.GainFilter3D`
    """

    inLightE = None
    """
    Receiver of the electric field required for computations [V/m].
    It is required only for the SHB computations.
    
    You will find usage details in the documentation of the receiver class
    :class:`~plask.flow.ModeLightEReceiver3D`.
    
    Example:
       Connect the reveiver to a provider from some other solver:
    
       >>> solver.inLightE = other_solver.outModeLightE
    
    See also:
    
       Receciver class: :class:`plask.flow.ModeLightEReceiver3D`
    
       Provider class: :class:`plask.flow.ModeLightEProvider3D`
    
       Data filter: :class:`plask.filter.ModeLightEFilter3D`
    """

    inTemperature = None
    """
    Receiver of the temperature required for computations [K].
    
    
    You will find usage details in the documentation of the receiver class
    :class:`~plask.flow.TemperatureReceiver3D`.
    
    Example:
       Connect the reveiver to a provider from some other solver:
    
       >>> solver.inTemperature = other_solver.outTemperature
    
    See also:
    
       Receciver class: :class:`plask.flow.TemperatureReceiver3D`
    
       Provider class: :class:`plask.flow.TemperatureProvider3D`
    
       Data filter: :class:`plask.filter.TemperatureFilter3D`
    """

    inWavelength = None
    """
    Receiver of the wavelength required for computations [nm].
    It is required only for the SHB computations.
    
    You will find usage details in the documentation of the receiver class
    :class:`~plask.flow.ModeWavelengthReceiver`.
    
    Example:
       Connect the reveiver to a provider from some other solver:
    
       >>> solver.inWavelength = other_solver.outModeWavelength
    
    See also:
    
       Receciver class: :class:`plask.flow.ModeWavelengthReceiver`
    
       Provider class: :class:`plask.flow.ModeWavelengthProvider`
    
       Data filter: :class:`plask.filter.ModeWavelengthFilter`
    """

    def initialize(self):
        """
        initialize(self)
        
            Initialize solver.
            
            This method manually initialized the solver and sets :attr:`initialized` to
            *True*. Normally calling it is not necessary, as each solver automatically
            initializes itself when needed.
            
            Returns:
                bool: solver :attr:`initialized` state prior to this method call.
        """

    initialized = None
    """
    True if the solver has been initialized. (read only)
    
    Solvers usually get initialized at the beginning of the computations.
    You can clean the initialization state and free the memory by calling
    the :meth:`invalidate` method.
    """

    def invalidate(self):
        """
        invalidate(self)
        
            Set the solver back to uninitialized state.
            
            This method frees the memory allocated by the solver and sets
            :attr:`initialized` to *False*.
        """

    iterative = None
    """
    Iterative matrix parameters (see :py:class:`~plask.IterativeParams`)
    """

    def load_xpl(self, xpl, manager):
        """
        load_xpl(self, xpl, manager)
        
            Load configuration from XPL reader.
            
            This method should be overriden in custom Python solvers.
            
            Example:
              .. code-block:: python
            
                 def load_xpl(self, xpl, manager):
                     for tag in xpl:
                         if tag == 'config':
                             self.a = tag['a']
                             self.b = tag.get('b', 0)
                             if 'c' in tag:
                                 self.c = tag['c']
                         if tag == 'combined':
                             for subtag in tag:
                                 if subtag == 'withtext':
                                     self.data = subtag.attrs
                                     # Text must be read last
                                     self.text = subtag.text
                         elif tag == 'geometry':
                             self.geometry = tag.getitem(manager.geo, 'ref')
        """

    maxerr = None
    """
    Maximum relative residual error (%)
    """

    mesh = None
    """
    Mesh provided to the solver
    """

    outCarriersConcentration = None
    """
    Provider of the computed carriers concentration [1/cm³].
    
    
    outCarriersConcentration(n=0, mesh, interpolation='default')
    
    :param str type: Detailed information which carriers are returned. It can be
                     'majority' to return majority carriers in given material,
                     'pairs' for the concentration of electron-hole pairs,
                     'electrons', or 'holes' for particular carriers type.
    :param mesh mesh: Target mesh to get the field at.
    :param str interpolation: Requested interpolation method.
    
    :return: Data with the carriers concentration on the specified mesh **[1/cm³]**.
    
    You may obtain the number of different values this provider can return by
    testing its length.
    
    Example:
       Connect the provider to a receiver in some other solver:
    
       >>> other_solver.inCarriersConcentration = solver.outCarriersConcentration
    
       Obtain the provided field:
    
       >>> solver.outCarriersConcentration(0, mesh)
       <plask.Data at 0x1234567>
    
       Test the number of provided values:
    
       >>> len(solver.outCarriersConcentration)
       3
    
    See also:
    
       Provider class: :class:`plask.flow.CarriersConcentrationProvider3D`
    
       Receciver class: :class:`plask.flow.CarriersConcentrationReceiver3D`
    """



class DiffusionCyl(plask.Solver):
    """
    DiffusionCyl(name="")
    
    Calculates carrier pairs concentration in active region using FEM in two-dimensional cylindrical space
    """
    algorithm = None
    """
    Chosen matrix factorization algorithm
    """

    def compute(self, loops=0, shb=False, reg=None):
        """
        compute(self, loops=0, shb=False, reg=None)
        
            Run diffusion calculations
            
            Args:
                loops (int): Number of iterations to perform. If 0, the solver will run
                             until the convergence.
                shb (bool): If True, the solver will use take into account the spatial hole
                            burning effect.
                reg (int or None): Index of the active region to compute. If None, perform
                                   computations for all the active regions.
        """

    def compute_overthreshold(self):
        """
        compute_overthreshold(self)
        
            Deprecated method. Use compute(shb=True) instead.
        """

    def compute_threshold(self):
        """
        compute_threshold(self)
        
            Deprecated method. Use compute() instead.
        """

    geometry = None
    """
    Geometry provided to the solver
    """

    def get_burning_for_mode(self, mode):
        """
        get_burning_for_mode(self, mode)
        
            Get power burned over threshold by specified mode [mW].
        """

    def get_total_burning(self):
        """
        get_total_burning(self)
        
            Get total power burned over threshold [mW].
        """

    id = None
    """
    Id of the solver object. (read only)
    
    Example:
        >>> mysolver.id
        mysolver:category.type
    """

    inCurrentDensity = None
    """
    Receiver of the current density required for computations [kA/cm²].
    
    
    You will find usage details in the documentation of the receiver class
    :class:`~plask.flow.CurrentDensityReceiverCyl`.
    
    Example:
       Connect the reveiver to a provider from some other solver:
    
       >>> solver.inCurrentDensity = other_solver.outCurrentDensity
    
    See also:
    
       Receciver class: :class:`plask.flow.CurrentDensityReceiverCyl`
    
       Provider class: :class:`plask.flow.CurrentDensityProviderCyl`
    
       Data filter: :class:`plask.filter.CurrentDensityFilterCyl`
    """

    inGain = None
    """
    Receiver of the material gain required for computations [1/cm].
    It is required only for the SHB computations.
    
    You will find usage details in the documentation of the receiver class
    :class:`~plask.flow.GainReceiverCyl`.
    
    Example:
       Connect the reveiver to a provider from some other solver:
    
       >>> solver.inGain = other_solver.outGain
    
    See also:
    
       Receciver class: :class:`plask.flow.GainReceiverCyl`
    
       Provider class: :class:`plask.flow.GainProviderCyl`
    
       Data filter: :class:`plask.filter.GainFilterCyl`
    """

    inLightE = None
    """
    Receiver of the electric field required for computations [V/m].
    It is required only for the SHB computations.
    
    You will find usage details in the documentation of the receiver class
    :class:`~plask.flow.ModeLightEReceiverCyl`.
    
    Example:
       Connect the reveiver to a provider from some other solver:
    
       >>> solver.inLightE = other_solver.outModeLightE
    
    See also:
    
       Receciver class: :class:`plask.flow.ModeLightEReceiverCyl`
    
       Provider class: :class:`plask.flow.ModeLightEProviderCyl`
    
       Data filter: :class:`plask.filter.ModeLightEFilterCyl`
    """

    inTemperature = None
    """
    Receiver of the temperature required for computations [K].
    
    
    You will find usage details in the documentation of the receiver class
    :class:`~plask.flow.TemperatureReceiverCyl`.
    
    Example:
       Connect the reveiver to a provider from some other solver:
    
       >>> solver.inTemperature = other_solver.outTemperature
    
    See also:
    
       Receciver class: :class:`plask.flow.TemperatureReceiverCyl`
    
       Provider class: :class:`plask.flow.TemperatureProviderCyl`
    
       Data filter: :class:`plask.filter.TemperatureFilterCyl`
    """

    inWavelength = None
    """
    Receiver of the wavelength required for computations [nm].
    It is required only for the SHB computations.
    
    You will find usage details in the documentation of the receiver class
    :class:`~plask.flow.ModeWavelengthReceiver`.
    
    Example:
       Connect the reveiver to a provider from some other solver:
    
       >>> solver.inWavelength = other_solver.outModeWavelength
    
    See also:
    
       Receciver class: :class:`plask.flow.ModeWavelengthReceiver`
    
       Provider class: :class:`plask.flow.ModeWavelengthProvider`
    
       Data filter: :class:`plask.filter.ModeWavelengthFilter`
    """

    def initialize(self):
        """
        initialize(self)
        
            Initialize solver.
            
            This method manually initialized the solver and sets :attr:`initialized` to
            *True*. Normally calling it is not necessary, as each solver automatically
            initializes itself when needed.
            
            Returns:
                bool: solver :attr:`initialized` state prior to this method call.
        """

    initialized = None
    """
    True if the solver has been initialized. (read only)
    
    Solvers usually get initialized at the beginning of the computations.
    You can clean the initialization state and free the memory by calling
    the :meth:`invalidate` method.
    """

    def invalidate(self):
        """
        invalidate(self)
        
            Set the solver back to uninitialized state.
            
            This method frees the memory allocated by the solver and sets
            :attr:`initialized` to *False*.
        """

    iterative = None
    """
    Iterative matrix parameters (see :py:class:`~plask.IterativeParams`)
    """

    def load_xpl(self, xpl, manager):
        """
        load_xpl(self, xpl, manager)
        
            Load configuration from XPL reader.
            
            This method should be overriden in custom Python solvers.
            
            Example:
              .. code-block:: python
            
                 def load_xpl(self, xpl, manager):
                     for tag in xpl:
                         if tag == 'config':
                             self.a = tag['a']
                             self.b = tag.get('b', 0)
                             if 'c' in tag:
                                 self.c = tag['c']
                         if tag == 'combined':
                             for subtag in tag:
                                 if subtag == 'withtext':
                                     self.data = subtag.attrs
                                     # Text must be read last
                                     self.text = subtag.text
                         elif tag == 'geometry':
                             self.geometry = tag.getitem(manager.geo, 'ref')
        """

    maxerr = None
    """
    Maximum relative residual error (%)
    """

    mesh = None
    """
    Mesh provided to the solver
    """

    outCarriersConcentration = None
    """
    Provider of the computed carriers concentration [1/cm³].
    
    
    outCarriersConcentration(n=0, mesh, interpolation='default')
    
    :param str type: Detailed information which carriers are returned. It can be
                     'majority' to return majority carriers in given material,
                     'pairs' for the concentration of electron-hole pairs,
                     'electrons', or 'holes' for particular carriers type.
    :param mesh mesh: Target mesh to get the field at.
    :param str interpolation: Requested interpolation method.
    
    :return: Data with the carriers concentration on the specified mesh **[1/cm³]**.
    
    You may obtain the number of different values this provider can return by
    testing its length.
    
    Example:
       Connect the provider to a receiver in some other solver:
    
       >>> other_solver.inCarriersConcentration = solver.outCarriersConcentration
    
       Obtain the provided field:
    
       >>> solver.outCarriersConcentration(0, mesh)
       <plask.Data at 0x1234567>
    
       Test the number of provided values:
    
       >>> len(solver.outCarriersConcentration)
       3
    
    See also:
    
       Provider class: :class:`plask.flow.CarriersConcentrationProviderCyl`
    
       Receciver class: :class:`plask.flow.CarriersConcentrationReceiverCyl`
    """

